@@ -15,14 +15,14 @@
 import ast
 import copy
 import json
-import netaddr
 import re
 
 from django.core.urlresolvers import reverse
 from django.core import validators as django_validator
 from django import forms
-from django.http import Http404
+from django.forms import widgets
 from django.template import defaultfilters
+from django.utils.encoding import force_text
 from django.utils import html
 from django.utils.translation import ugettext_lazy as _
 from horizon import exceptions
@@ -31,15 +31,14 @@ from horizon import messages
 from openstack_dashboard.api import glance
 from openstack_dashboard.api import nova
 from oslo_log import log as logging
-import yaql
+from oslo_log import versionutils
+import six
+from yaql import legacy
 
-from muranoclient.common import exceptions as muranoclient_exc
 from muranodashboard.api import packages as pkg_api
 from muranodashboard.common import net
 from muranodashboard.environments import api as env_api
 
-from oslo_log import versionutils
-
 
 LOG = logging.getLogger(__name__)
 
@@ -52,7 +51,7 @@ def with_request(func):
     """
     def update(self, initial, request=None, **kwargs):
         initial_request = initial.get('request')
-        for key, value in initial.iteritems():
+        for key, value in six.iteritems(initial):
             if key != 'request' and key not in kwargs:
                 kwargs[key] = value
 
@@ -75,7 +74,7 @@ def make_yaql_validator(validator_property):
     message = validator_property.get('message', '')
 
     def validator_func(value):
-        context = yaql.create_context()
+        context = legacy.create_context()
         context['$'] = value
         if not expr.evaluate(context=context):
             raise forms.ValidationError(message)
@@ -135,17 +134,23 @@ class RawProperty(object):
     def __init__(self, key, spec):
         self.key = key
         self.spec = spec
+        self.value = None
+        self.value_evaluated = False
 
-    def finalize(self, form_name, service):
+    def finalize(self, form_name, service, cls):
         def _get(field):
-            data_ready, value = service.get_data(form_name, self.spec)
-            return value if data_ready else field.__dict__[self.key]
+            if self.value_evaluated:
+                return self.value
+            return service.get_data(form_name, self.spec)
 
         def _set(field, value):
-            field.__dict__[self.key] = value
+            self.value = value
+            self.value_evaluated = value is not None
+            if hasattr(cls, self.key):
+                getattr(cls, self.key).fset(field, value)
 
         def _del(field):
-            del field.__dict__[self.key]
+            _set(field, None)
         return property(_get, _set, _del)
 
 
@@ -157,11 +162,11 @@ class CustomPropertiesField(forms.Field):
                  *args, **kwargs):
         self.description = description
         self.description_title = (description_title or
-                                  unicode(kwargs.get('label', '')))
+                                  force_text(kwargs.get('label', '')))
 
         for arg in FIELD_ARGS_TO_ESCAPE:
             if kwargs.get(arg):
-                kwargs[arg] = html.escape(unicode(kwargs[arg]))
+                kwargs[arg] = html.escape(force_text(kwargs[arg]))
 
         validators = []
         for validator in kwargs.get('validators', []):
@@ -194,9 +199,10 @@ class CustomPropertiesField(forms.Field):
     @classmethod
     def finalize_properties(cls, kwargs, form_name, service):
         props = {}
-        for key, value in kwargs.items():
+        kwargs_ = copy.copy(kwargs)
+        for key, value in kwargs_.items():
             if isinstance(value, RawProperty):
-                props[key] = value.finalize(form_name, service)
+                props[key] = value.finalize(form_name, service, cls)
                 del kwargs[key]
         if props:
             return type(cls.__name__, (cls,), props)
@@ -209,11 +215,12 @@ class CharField(forms.CharField, CustomPropertiesField):
 
 
 class PasswordField(CharField):
-    special_characters = '!@#$%^&*()_+|\/.,~?><:{}'
+    special_characters = '!@#$%^&*()_+|\/.,~?><:{}-'
     password_re = re.compile('^.*(?=.*\d)(?=.*[a-z])(?=.*[A-Z])(?=.*[%s]).*$'
                              % special_characters)
     has_clone = False
     original = True
+    attrs = {'data-type': 'password'}
     validate_password = django_validator.RegexValidator(
         password_re, _('The password must contain at least one letter, one   \
                                number and one special character'), 'invalid')
@@ -235,27 +242,39 @@ class PasswordField(CharField):
             js = ('muranodashboard/js/passwordfield.js',)
 
     def __init__(self, label, *args, **kwargs):
+        self.confirm_input = kwargs.pop('confirm_input', True)
+
+        kwargs.update({'label': label,
+                       'error_messages': kwargs.get('error_messages', {}),
+                       'widget': forms.PasswordInput(attrs=self.attrs,
+                                                     render_value=True)})
+
+        validators = kwargs.get('validators')
         help_text = kwargs.get('help_text')
-        if not help_text:
-            help_text = _('Enter a complex password with at least one letter, \
-                one number and one special character')
 
-        error_messages = {
-            'invalid': self.validate_password.message}
-        err_msg = kwargs.get('error_messages')
-        if err_msg:
-            if err_msg.get('required'):
-                error_messages['required'] = err_msg.get('required')
+        if not validators:
+            # No custom validators, using default validator
+            validators = [self.validate_password]
+            if not help_text:
+                help_text = _(
+                    'Enter a complex password with at least one letter, '
+                    'one number and one special character')
 
-        kwargs.update({
-            'min_length': 7,
-            'max_length': 255,
-            'validators': [self.validate_password],
-            'label': label,
-            'error_messages': error_messages,
-            'help_text': help_text,
-            'widget': self.PasswordInput(render_value=True),
-        })
+            kwargs['error_messages'].setdefault(
+                'invalid', self.validate_password.message)
+            kwargs['min_length'] = kwargs.get('min_length', 7)
+            kwargs['max_length'] = kwargs.get('max_length', 255)
+            kwargs['widget'] = self.PasswordInput(attrs=self.attrs,
+                                                  render_value=True)
+        else:
+            if not help_text:
+                # NOTE(kzaitsev) There are custom validators for password,
+                # but no help text let's leave only a generic message,
+                # since we do not know exact constraints
+                help_text = _('Enter a password')
+
+        kwargs.update({'validators': validators,
+                       'help_text': help_text})
 
         super(PasswordField, self).__init__(*args, **kwargs)
 
@@ -280,7 +299,13 @@ class IntegerField(forms.IntegerField, CustomPropertiesField):
 
 
 class ChoiceField(forms.ChoiceField, CustomPropertiesField):
-    pass
+    def __init__(self, **kwargs):
+        choices = kwargs.get('choices') or getattr(self, 'choices', None)
+        if choices:
+            if isinstance(choices, dict):
+                choices = list(choices.items())
+            kwargs['choices'] = choices
+        super(ChoiceField, self).__init__(**kwargs)
 
 
 class DynamicChoiceField(hz_forms.DynamicChoiceField, CustomPropertiesField):
@@ -295,12 +320,12 @@ class FlavorChoiceField(ChoiceField):
 
     @with_request
     def update(self, request, **kwargs):
-        self.choices = []
+        choices = []
         flavors = nova.novaclient(request).flavors.list()
 
         # If no requirements are present, return all the flavors.
         if not hasattr(self, 'requirements'):
-            self.choices = [(flavor.name, flavor.name) for flavor in flavors]
+            choices = [(flavor.name, flavor.name) for flavor in flavors]
         else:
             for flavor in flavors:
                 # If a flavor doesn't meet a minimum requirement,
@@ -312,12 +337,31 @@ class FlavorChoiceField(ChoiceField):
                     continue
                 if flavor.ram < self.requirements.get('min_memory_mb', 0):
                     continue
-                self.choices.append((flavor.name, flavor.name))
-        # Search through selected flavors
-        for flavor_name, flavor_name in self.choices:
-            if 'medium' in flavor_name:
-                self.initial = flavor_name
-                break
+                if 'max_vcpus' in self.requirements:
+                    if flavor.vcpus > self.requirements['max_vcpus']:
+                        continue
+                if 'max_disk' in self.requirements:
+                    if flavor.disk > self.requirements['max_disk']:
+                        continue
+                if 'max_memory_mb' in self.requirements:
+                    if flavor.ram > self.requirements['max_memory_mb']:
+                        continue
+                choices.append((flavor.name, flavor.name))
+
+        choices.sort(key=lambda e: e[1])
+        self.choices = choices
+        if kwargs.get('form'):
+            kwargs_form_flavor = kwargs["form"].fields.get('flavor')
+        else:
+            kwargs_form_flavor = None
+        if kwargs_form_flavor:
+            self.initial = kwargs["form"]["flavor"].value()
+        else:
+            # Search through selected flavors
+            for flavor_name, flavor_name in self.choices:
+                if 'medium' in flavor_name:
+                    self.initial = flavor_name
+                    break
 
 
 class KeyPairChoiceField(DynamicChoiceField):
@@ -325,7 +369,9 @@ class KeyPairChoiceField(DynamicChoiceField):
     @with_request
     def update(self, request, **kwargs):
         self.choices = [('', _('No keypair'))]
-        for keypair in nova.novaclient(request).keypairs.list():
+        for keypair in sorted(
+                nova.novaclient(request).keypairs.list(),
+                key=lambda e: e.name):
             self.choices.append((keypair.name, keypair.name))
 
 
@@ -384,7 +430,8 @@ class ImageChoiceField(ChoiceField):
                     continue
             image_map[image.id] = title
 
-        for id_, title in sorted(image_map.iteritems(), key=lambda e: e[1]):
+        for id_, title in sorted(six.iteritems(image_map),
+                                 key=lambda e: e[1].title):
             image_choices.append((id_, title))
         if image_choices:
             image_choices.insert(0, ("", _("Select Image")))
@@ -454,6 +501,7 @@ class AZoneChoiceField(ChoiceField):
         if not az_choices:
             az_choices.insert(0, ("", _("No availability zones available")))
 
+        az_choices.sort(key=lambda e: e[1])
         self.choices = az_choices
 
 
@@ -478,86 +526,9 @@ class FloatingIpBooleanField(BooleanField):
     pass
 
 
-class ClusterIPField(CharField):
-    existing_subnet = None
-    network_topology = None
-    router_id = None
-
-    @staticmethod
-    def make_nova_validator(request, ip_ranges):
-        def perform_checking(ip):
-            django_validator.validate_ipv4_address(ip)
-            if not netaddr.all_matching_cidrs(ip, ip_ranges) and ip_ranges:
-                raise forms.ValidationError(_('Specified Cluster Static IP is'
-                                              ' not in valid IP range'))
-            try:
-                ip_info = nova.novaclient(request).fixed_ips.get(ip)
-            except exceptions.UNAUTHORIZED:
-                LOG.error("Error to get information about IP address"
-                          " using novaclient")
-                exceptions.handle(
-                    request, _("Unable to retrieve information "
-                               "about fixed IP or IP is not valid."),
-                    ignore=True)
-            except exceptions.NOT_FOUND:
-                msg = "Could not found fixed ips for ip %s" % (ip,)
-                LOG.error(msg)
-                exceptions.handle(
-                    request, msg,
-                    ignore=True)
-            else:
-                if ip_info.hostname:
-                    raise forms.ValidationError(
-                        _('Specified Cluster Static IP is already in use'))
-        return perform_checking
-
-    def update_network_params(self, request, environment_id):
-        env = env_api.environment_get(request, environment_id)
-        self.existing_subnet = env.networking.get('cidr')
-        self.network_topology = env.networking.get('topology')
-
-    def make_neutron_validator(self):
-        def perform_checking(ip):
-            django_validator.validate_ipv4_address(ip)
-            if not self.existing_subnet:
-                raise forms.ValidationError(
-                    _('Cannot get allowed subnet for the environment, '
-                      'consult your admin'))
-            elif not netaddr.IPAddress(ip) in netaddr.IPNetwork(
-                    self.existing_subnet):
-                raise forms.ValidationError(
-                    _('Specified IP address should belong to {0} '
-                      'subnet').format(self.existing_subnet))
-
-        return perform_checking
-
-    @with_request
-    def update(self, request, environment_id, **kwargs):
-        self.update_network_params(request, environment_id)
-
-        if self.network_topology == 'nova':
-            try:
-                network_list = nova.novaclient(request).networks.list()
-                ip_ranges = [network.cidr for network in network_list]
-                ranges = ', '.join(ip_ranges)
-            except StandardError:
-                ip_ranges, ranges = [], ''
-            if ip_ranges:
-                self.help_text = _('Select IP from '
-                                   'available range: {0} ').format(ranges)
-            else:
-                self.help_text = _('Specify valid fixed IP')
-            self.validators = [self.make_nova_validator(request, ip_ranges)]
-        elif self.network_topology in ('routed', 'manual'):
-            if self.network_topology == 'manual' and self.router_id is None:
-                raise muranoclient_exc.NotFound(_(
-                    'Router is not found. You should create one explicitly.'))
-            self.widget.attrs['placeholder'] = self.existing_subnet
-            self.validators = [self.make_neutron_validator()]
-        else:  # 'flat' topology
-            raise NotImplementedError('Flat topology is not implemented yet')
-        self.error_messages['invalid'] = \
-            django_validator.validate_ipv4_address.message
+class ClusterIPField(forms.GenericIPAddressField, CustomPropertiesField):
+    def __init__(self, *args, **kwargs):
+        super(ClusterIPField, self).__init__(protocol='ipv4', *args, **kwargs)
 
 
 class DatabaseListField(CharField):
@@ -582,24 +553,37 @@ class DatabaseListField(CharField):
             self.validate_mssql_identifier(db_name)
 
 
+class ErrorWidget(widgets.Widget):
+
+    def __init__(self, *args, **kwargs):
+        self.message = kwargs.pop(
+            'message', _("There was an error initialising this field."))
+        super(ErrorWidget, self).__init__(*args, **kwargs)
+
+    def render(self, name, value, attrs=None):
+        return "<div name={name}>{message}</div>".format(
+            name=name, message=self.message)
+
+
+class MuranoTypeWidget(hz_forms.fields.DynamicSelectWidget):
+    def __init__(self, attrs=None, **kwargs):
+        if attrs is None:
+            attrs = {'class': 'murano_add_select'}
+        else:
+            attrs.setdefault('class', '')
+            attrs['class'] += ' murano_add_select'
+        super(MuranoTypeWidget, self).__init__(attrs=attrs, **kwargs)
+
+    class Media(object):
+        js = ('muranodashboard/js/add-select.js',)
+
+
 def make_select_cls(fqns):
     if not isinstance(fqns, (tuple, list)):
         fqns = (fqns,)
 
-    class Widget(hz_forms.fields.DynamicSelectWidget):
-        def __init__(self, attrs=None, **kwargs):
-            if attrs is None:
-                attrs = {'class': 'murano_add_select'}
-            else:
-                attrs.setdefault('class', '')
-                attrs['class'] += ' murano_add_select'
-            super(Widget, self).__init__(attrs=attrs, **kwargs)
-
-        class Media(object):
-            js = ('muranodashboard/js/add-select.js',)
-
     class DynamicSelect(hz_forms.DynamicChoiceField, CustomPropertiesField):
-        widget = Widget
+        widget = MuranoTypeWidget
 
         def __init__(self, empty_value_message=None, *args, **kwargs):
             super(DynamicSelect, self).__init__(*args, **kwargs)
@@ -610,21 +594,52 @@ def make_select_cls(fqns):
 
         @with_request
         def update(self, request, environment_id, **kwargs):
-            def _make_link():
-                ns_url = 'horizon:murano:catalog:add'
+            matching_classes = []
+            fqns_seen = set()
+            # NOTE(kzaitsev): it's possible to have a private
+            # and public apps with the same fqn, however the engine would
+            # currently favor private package. Therefore we should squash
+            # these until we devise a better way to work with this
+            # situation and versioning
 
-                def _reverse(_fqn):
-                    _app = pkg_api.app_by_fqn(request, _fqn)
-                    if _app is None:
-                        msg = "Application with FQN='{0}' doesn't exist"
-                        messages.error(request, msg.format(_fqn))
-                        raise Http404(msg.format(_fqn))
-                    args = (_app.id, environment_id, False, True)
-                    return _app.name, reverse(ns_url, args=args)
-                return json.dumps([_reverse(cls) for cls in fqns])
+            for class_fqn in fqns:
+                app_found = pkg_api.app_by_fqn(request, class_fqn)
+                if app_found:
+                    fqns_seen.add(app_found.fully_qualified_name)
+                    matching_classes.append(app_found)
+
+                apps_found = pkg_api.apps_that_inherit(request, class_fqn)
+                for app in apps_found:
+                    if app.fully_qualified_name in fqns_seen:
+                        continue
+                    fqns_seen.add(app.fully_qualified_name)
+                    matching_classes.append(app)
+
+            if not matching_classes:
+                msg = _(
+                    "Couldn't find any apps, required for this field.\n"
+                    "Tried: {fqns}").format(fqns=', '.join(fqns))
+                self.widget = ErrorWidget(message=msg)
+
+            # NOTE(kzaitsev): this closure is needed to allow us have custom
+            # logic when clicking add button
+            def _make_link():
+                ns_url = 'horizon:app-catalog:catalog:add'
+                ns_url_args = (environment_id, False, True)
+
+                # This will prevent horizon from adding an extra '+' button
+                if not matching_classes:
+                    return ''
+
+                return json.dumps([
+                    (app.name, reverse(ns_url, args=((app.id,) + ns_url_args)))
+                    for app in matching_classes])
 
             self.widget.add_item_link = _make_link
-            apps = env_api.service_list_by_fqns(request, environment_id, fqns)
+
+            apps = env_api.service_list_by_fqns(
+                request, environment_id,
+                [app.fully_qualified_name for app in matching_classes])
             choices = [('', self.empty_value_message)]
             choices.extend([(app['?']['id'],
                              html.escape(app.name)) for app in apps])
