@@ -20,10 +20,12 @@ from horizon import exceptions
 from horizon import messages
 from horizon import tables
 from horizon.utils import filters
+from openstack_dashboard import policy
 from oslo_log import log as logging
 
 from muranoclient.common import exceptions as exc
 from muranodashboard import api
+from muranodashboard.common import utils as md_utils
 
 LOG = logging.getLogger(__name__)
 
@@ -31,17 +33,19 @@ LOG = logging.getLogger(__name__)
 class ImportBundle(tables.LinkAction):
     name = 'import_bundle'
     verbose_name = _('Import Bundle')
-    url = 'horizon:murano:packages:import_bundle'
+    url = 'horizon:app-catalog:packages:import_bundle'
     classes = ('ajax-modal',)
     icon = "plus"
+    policy_rules = (("murano", "upload_package"),)
 
 
 class ImportPackage(tables.LinkAction):
     name = 'upload_package'
     verbose_name = _('Import Package')
-    url = 'horizon:murano:packages:upload'
+    url = 'horizon:app-catalog:packages:upload'
     classes = ('ajax-modal',)
     icon = "plus"
+    policy_rules = (("murano", "upload_package"),)
 
     def allowed(self, request, package):
         _allowed = False
@@ -62,7 +66,8 @@ class PackagesFilterAction(tables.FilterAction):
 class DownloadPackage(tables.LinkAction):
     name = 'download_package'
     verbose_name = _('Download Package')
-    url = 'horizon:murano:packages:download'
+    policy_rules = (("murano", "download_package"),)
+    url = 'horizon:app-catalog:packages:download'
 
     def allowed(self, request, package):
         return True
@@ -76,6 +81,7 @@ class ToggleEnabled(tables.BatchAction):
     name = 'toggle_enabled'
     verbose_name = _("Toggle Enabled")
     icon = "toggle-on"
+    policy_rules = (("murano", "modify_package"),)
 
     @staticmethod
     def action_present(count):
@@ -104,13 +110,14 @@ class ToggleEnabled(tables.BatchAction):
             exceptions.handle(
                 request,
                 msg,
-                redirect=reverse('horizon:murano:packages:index'))
+                redirect=reverse('horizon:app-catalog:packages:index'))
 
 
 class TogglePublicEnabled(tables.BatchAction):
     name = 'toggle_public_enabled'
     verbose_name = _("Toggle Public")
     icon = "share-alt"
+    policy_rules = (("murano", "publicize_package"),)
 
     @staticmethod
     def action_present(count):
@@ -139,7 +146,7 @@ class TogglePublicEnabled(tables.BatchAction):
             exceptions.handle(
                 request,
                 msg,
-                redirect=reverse('horizon:murano:packages:index'))
+                redirect=reverse('horizon:app-catalog:packages:index'))
         except exc.HTTPConflict:
             msg = _('Package or Class with the same name is already made '
                     'public')
@@ -148,11 +155,12 @@ class TogglePublicEnabled(tables.BatchAction):
             exceptions.handle(
                 request,
                 msg,
-                redirect=reverse('horizon:murano:packages:index'))
+                redirect=reverse('horizon:app-catalog:packages:index'))
 
 
-class DeletePackage(tables.DeleteAction):
+class DeletePackage(policy.PolicyTargetMixin, tables.DeleteAction):
     name = 'delete_package'
+    policy_rules = (("murano", "delete_package"),)
 
     @staticmethod
     def action_present(count):
@@ -179,16 +187,16 @@ class DeletePackage(tables.DeleteAction):
             exceptions.handle(
                 self.request,
                 msg,
-                redirect=reverse('horizon:murano:packages:index'))
+                redirect=reverse('horizon:app-catalog:packages:index'))
         except exc.HTTPForbidden:
             msg = _("You are not allowed to delete this package")
             LOG.exception(msg)
             exceptions.handle(
                 request, msg,
-                redirect=reverse('horizon:murano:packages:index'))
+                redirect=reverse('horizon:app-catalog:packages:index'))
         except Exception:
             LOG.exception(_('Unable to delete package in murano-api server'))
-            url = reverse('horizon:murano:packages:index')
+            url = reverse('horizon:app-catalog:packages:index')
             exceptions.handle(request,
                               _('Unable to remove package.'),
                               redirect=url)
@@ -197,18 +205,20 @@ class DeletePackage(tables.DeleteAction):
 class ModifyPackage(tables.LinkAction):
     name = 'modify_package'
     verbose_name = _('Modify Package')
-    url = 'horizon:murano:packages:modify'
+    url = 'horizon:app-catalog:packages:modify'
     classes = ('ajax-modal',)
     icon = "edit"
+    policy_rules = (("murano", "modify_package"),)
 
     def allowed(self, request, package):
         return True
 
 
 class PackageDefinitionsTable(tables.DataTable):
-    name = tables.Column('name',
-                         link="horizon:murano:packages:detail",
-                         verbose_name=_('Package Name'))
+    name = md_utils.Column(
+        'name',
+        link="horizon:app-catalog:packages:detail",
+        verbose_name=_('Package Name'))
     tenant_name = tables.Column('tenant_name', verbose_name=_('Tenant Name'))
     enabled = tables.Column('enabled', verbose_name=_('Active'))
     is_public = tables.Column('is_public', verbose_name=_('Public'))
