@@ -437,13 +437,15 @@ class ImageTestCase(PackageBase):
         glance_endpoint = cls.service_catalog.url_for(service_type='image')
         cls.glance = gclient.Client('1', endpoint=glance_endpoint,
                                     session=cls.keystone_client.session)
-        cls.image_title = 'New Image ' + str(time.time())
-        cls.image = cls.upload_image(cls.image_title)
 
-    @classmethod
-    def tearDownClass(cls):
-        super(ImageTestCase, cls).tearDownClass()
-        cls.glance.images.delete(cls.image.id)
+    def setUp(self):
+        super(ImageTestCase, self).setUp()
+        self.image_title = self.gen_random_resource_name('test-image', 15)
+        self.image = self.upload_image(self.image_title)
+
+    def tearDown(self):
+        super(ImageTestCase, self).tearDown()
+        self.glance.images.delete(self.image.id)
 
     @classmethod
     def upload_image(cls, title):
@@ -464,16 +466,6 @@ class ImageTestCase(PackageBase):
         self.driver.find_element_by_xpath(
             ".//*[@value = '{0}']".format(element)).click()
 
-    def repair_image(self):
-        self.driver.find_element_by_id(
-            'marked_images__action_mark_image').click()
-        self.select_from_list('image', self.image.id)
-        self.fill_field(by.By.ID, 'id_title', self.image_title)
-        self.select_from_list('type', 'linux')
-        self.select_and_click_element('Mark Image')
-        self.check_element_on_page(by.By.XPATH,
-                                   consts.TestImage.format(self.image_title))
-
 
 class FieldsTestCase(PackageBase):
     def check_error_message_is_present(self, error_message):
@@ -499,12 +491,12 @@ class ApplicationTestCase(ImageTestCase):
     def delete_component(self, component_name=None):
         if component_name:
             component_id = self.get_element_id(component_name)
-            btn = self.driver.find_element_by_id(
-                'services__row_{0}__action_delete'.format(
+            btn = self.wait_element_is_clickable(
+                by.By.ID, 'services__row_{0}__action_delete'.format(
                     component_id))
         else:
-            btn = self.driver.find_element_by_css_selector(
-                consts.DeleteComponent)
+            btn = self.wait_element_is_clickable(by.By.CSS_SELECTOR,
+                                                 consts.DeleteComponent)
         btn.click()
         el = self.wait_element_is_clickable(by.By.LINK_TEXT,
                                             'Delete Component', sec=30)
@@ -560,13 +552,17 @@ class ApplicationTestCase(ImageTestCase):
         self.select_and_click_action_for_app(action, app)
         field_id = "{0}_0-name".format(app_id)
         self.fill_field(by.By.ID, field_id, value=app_name)
-        self.driver.find_element_by_xpath(consts.ButtonSubmit).click()
-        self.driver.find_element_by_xpath(consts.InputSubmit).click()
+        self.wait_element_is_clickable(by.By.XPATH,
+                                       consts.ButtonSubmit).click()
+        self.wait_element_is_clickable(by.By.XPATH,
+                                       consts.InputSubmit).click()
         self.select_from_list('osImage', self.image.id)
 
-        self.driver.find_element_by_xpath(consts.InputSubmit).click()
+        self.wait_element_is_clickable(by.By.XPATH,
+                                       consts.InputSubmit).click()
         if env_id:
-            self.driver.find_element_by_xpath(consts.InputSubmit).click()
+            self.wait_element_is_clickable(by.By.XPATH,
+                                           consts.InputSubmit).click()
             self.wait_element_is_clickable(by.By.ID, consts.AddComponent)
             self.check_element_on_page(by.By.LINK_TEXT, app_name)
         else:
